@@ -179,10 +179,12 @@ class MSLDAPClient:
 			yield MSADUser.from_ldap(entry, self._ldapinfo), None
 		logger.debug('Finished polling for entries!')
 
-	async def get_all_machines(self):
+	async def get_all_machines(self, attrs = MSADMachine_ATTRS):
 		"""
 		Fetches all machine objects available in the LDAP tree and yields them as MSADMachine object.
 
+		:param attrs: Lists of attributes to request (eg. `['sAMAccountName', 'dNSHostName']`) Default: all attrs.
+		:type attrs: list
 		:return: Async generator which yields (`MSADMachine`, None) tuple on success or (None, `Exception`) on error
 		:rtype: Iterator[(:class:`MSADMachine`, :class:`Exception`)]
 		
@@ -190,7 +192,7 @@ class MSLDAPClient:
 		logger.debug('Polling AD for all user objects')
 		ldap_filter = r'(sAMAccountType=805306369)'
 
-		async for entry, err in self.pagedsearch(ldap_filter, MSADMachine_ATTRS):
+		async for entry, err in self.pagedsearch(ldap_filter, attrs):
 			if err is not None:
 				yield None, err
 				return
@@ -226,6 +228,60 @@ class MSLDAPClient:
 		async for entry, err in self.pagedsearch(ldap_filter, attributes):
 			yield entry, err
 
+	async def get_schemaentry(self, dn):
+		"""
+		Fetches one Schema entriy identified by dn
+
+		:return: (`MSADSchemaEntry`, None) tuple on success or (None, `Exception`) on error
+		:rtype: (:class:`MSADSchemaEntry`, :class:`Exception`)
+		"""
+		logger.debug('Polling Schema entry for %s'% dn)
+		
+		async for entry, err in self._con.pagedsearch(
+			dn, 
+			r'(distinguishedName=%s)' % escape_filter_chars(dn),
+			attributes = [x.encode() for x in MSADSCHEMAENTRY_ATTRS], 
+			size_limit = self.ldap_query_page_size, 
+			search_scope=BASE, 
+			controls = None, 
+			):
+				if err is not None:
+					raise err
+		
+				return MSADSchemaEntry.from_ldap(entry), None
+		else:
+			return None, None
+		logger.debug('Finished polling for entries!')
+	
+	async def get_all_schemaentry(self):
+		"""
+		Fetches all Schema entries under CN=Schema,CN=Configuration,...
+
+		:return: Async generator which yields (`MSADSchemaEntry`, None) tuple on success or (None, `Exception`) on error
+		:rtype: Iterator[(:class:`MSADSchemaEntry`, :class:`Exception`)]
+		"""
+		res = await self.get_tree_plot('CN=Schema,CN=Configuration,' + self._tree, level = 1)		
+		for x in res:
+			for dn in res[x]:
+				async for entry, err in self._con.pagedsearch(
+					dn, 
+					r'(distinguishedName=%s)' % escape_filter_chars(dn),
+					attributes = [x.encode() for x in MSADSCHEMAENTRY_ATTRS], 
+					size_limit = self.ldap_query_page_size, 
+					search_scope=BASE, 
+					controls = None, 
+					):
+						if err is not None:
+							yield None, err
+							return
+									
+						yield MSADSchemaEntry.from_ldap(entry), None
+						break
+				else:
+					yield None, None
+					
+		logger.debug('Finished polling for entries!')
+
 	async def get_laps(self, sAMAccountName):
 		"""
 		Fetches the LAPS password for a machine. This functionality is only available to specific high-privileged users.
@@ -260,6 +316,25 @@ class MSLDAPClient:
 			return None, None
 		logger.debug('Finished polling for entries!')
 
+	async def get_machine(self, sAMAccountName):
+		"""
+		Fetches one machine object from the AD, based on the sAMAccountName attribute (read: username) 
+		
+		:param sAMAccountName: The username of the machine.
+		:type sAMAccountName: str
+		:return: A tuple with the user as `MSADMachine` and an `Exception` is there was any
+		:rtype: (:class:`MSADMachine`, :class:`Exception`)
+		"""
+		logger.debug('Polling AD for user %s'% sAMAccountName)
+		ldap_filter = r'(&(sAMAccountType=805306369)(sAMAccountName=%s))' % sAMAccountName
+		async for entry, err in self.pagedsearch(ldap_filter, MSADMachine_ATTRS):
+			if err is not None:
+				return None, err
+			return MSADMachine.from_ldap(entry, self._ldapinfo), None
+		else:
+			return None, None
+		logger.debug('Finished polling for entries!')
+
 	async def get_ad_info(self):
 		"""
 		Polls for basic AD information (needed for determine password usage characteristics!)
@@ -414,12 +489,18 @@ class MSLDAPClient:
 		"""
 		
 		req_flags = SDFlagsRequestValue({'Flags' : flags})
-		controls = [('1.2.840.113556.1.4.801', True, req_flags.dump())]
+		controls = [
+					Control({
+						'controlType' : b'1.2.840.113556.1.4.801',
+						'controlValue': req_flags.dump(),
+						'criticality' : True,
+					})
+				]
 
 		changes = {
 			'nTSecurityDescriptor': [('replace', [data])]
 		}
-		return await self._con.modify(object_dn, changes, controls = controls)		
+		return await self._con.modify(object_dn, changes, controls = controls)	
 		
 	async def get_all_groups(self):
 		"""
@@ -833,6 +914,25 @@ class MSLDAPClient:
 		}
 		return await self._con.modify(group_dn, changes)
 
+	async def del_user_from_group(self, user_dn: str, group_dn: str):
+		"""
+		Removes user from group
+
+		:param user_dn: The user's DN
+		:type user_dn: str
+		:param group_dn: The groups's DN
+		:type group_dn: str
+		:return: A tuple of (True, None) on success or (False, Exception) on error. 
+		:rtype: (:class:`bool`, :class:`Exception`)
+
+
+		"""
+		changes = {
+			'member': [('delete', [user_dn])]
+		}
+		return await self._con.modify(group_dn, changes)
+		
+
 	async def get_object_by_dn(self, dn, expected_class = None):
 		ldap_filter = r'(distinguishedName=%s)' % dn
 		async for entry, err in self.pagedsearch(ldap_filter, ALL_ATTRIBUTES):
@@ -863,7 +963,12 @@ class MSLDAPClient:
 		:return: A tuple of (True, None) on success or (False, Exception) on error. 
 		:rtype: (:class:`bool`, :class:`Exception`)
 		"""
-		return await self._con.modify(dn, changes, controls=controls)
+		if controls is None:
+			controls = []
+		controls_conv = []
+		for control in controls:		
+			controls_conv.append(Control(control))
+		return await self._con.modify(dn, changes, controls=controls_conv)
 
 
 	async def add(self, dn, attributes):
@@ -1024,4 +1129,4 @@ class MSLDAPClient:
 	#	logger.debug('Polling AD for all user objects')
 	#	ldap_filter = r'(sAMAccountType=805306368)'
 	#
-	#	return self.pagedsearch(ldap_filter, MSADUser_ATTRS)
\ No newline at end of file
+	#	return self.pagedsearch(ldap_filter, MSADUser_ATTRS)
