@@ -23,6 +23,8 @@ from minikerberos.gssapi.channelbindings import ChannelBindingsStruct
 
 class MSLDAPClientConnection:
 	def __init__(self, target, creds):
+		if target is None:
+			raise Exception('Target cant be none!')
 		self.target = target
 		self.creds = creds
 		self.auth = AuthenticatorBuilder(self.creds, self.target).build()
@@ -198,8 +200,10 @@ class MSLDAPClientConnection:
 
 		logger.debug('Disconnecting!')
 		self.bind_ok = False
-		self.handle_incoming_task.cancel()
-		await self.network.terminate()
+		if self.handle_incoming_task is not None:
+			self.handle_incoming_task.cancel()
+		if self.network is not None:
+			await self.network.terminate()
 
 
 	def __bind_success(self):
@@ -463,7 +467,7 @@ class MSLDAPClientConnection:
 		:param changes: Describes the changes to be made on the object. Must be a dictionary of the following format: {'attribute': [('change_type', [value])]}
 		:type changes: dict
 		:param controls: additional controls to be passed in the query
-		:type controls: dict
+		:type controls: List[class:`Control`] 
 		:return: A tuple of (True, None) on success or (False, Exception) on error. 
 		:rtype: (:class:`bool`, :class:`Exception`)
 		"""
@@ -549,7 +553,7 @@ class MSLDAPClientConnection:
 		:param timeLimit: Maximum time the search should take. If time limit reached the server SHOULD return an error
 		:type timeLimit: int
 		:param controls: additional controls to be passed in the query
-		:type controls: dict
+		:type controls: List[class:`Control`]
 		:param return_done: Controls wether the final 'done' LDAP message should be returned, or just the actual results
 		:type return_done: bool
 
@@ -589,8 +593,6 @@ class MSLDAPClientConnection:
 					msg_type = message['protocolOp'].name
 					message = message.native
 					if msg_type == 'searchResDone':
-						#print(message)
-						#print('BREAKING!')
 						if return_done is True:
 							yield (message, None)
 						break
@@ -676,7 +678,7 @@ class MSLDAPClientConnection:
 						if err is not None:
 							yield (None, err)
 							return
-
+						
 						if 'resultCode' in res['protocolOp']:
 							for control in res['controls']:
 								if control['controlType'] == b'1.2.840.113556.1.4.319':
